/*  Orphy GTS acquisition library (liborphy)
 *  Copyright (c) Clment 'phneutre' Bourdarias
 *                   email: phneutre@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "orphy.h"

/* different EAs have different multiplication factors */
/* these values come from Orphy GTS user's manual */
float OGTS_EA_factors[8] = {/*EA 0*/5, 5, 5, 5, 0.1, 0.1, 1, /*EA 7*/2};
char OGTS_EAs[8] = {OGTS_EA0,OGTS_EA1,OGTS_EA2,OGTS_EA3,
		    OGTS_EA4,OGTS_EA5,OGTS_EA6,OGTS_EA7};

extern struct timeval start;

int stop_flag=0;
int wait_flag=1;
int time_i=0,time_f=0;
char stopacq = 0;

/*
==================
OGTS_CountEAs
==================
*/

int OGTS_CountEAs(OGTS_pref_t *prefs)
{
  int i=0,j=0;

  for (j=0; j<8; j++) {
    if (prefs->EA & OGTS_EAs[j]) {i++;}
  }

  return i;
}

/*
==================
OGTS_StartEA

- param: number of acq to do if mode == OGTS_NUM,
         total time of acq   if mode == OGTS_TIME,
         key to press to stop acq if mode == OGTS_KEY

This function is mainly used in threads-enabled applications,
or in applications that do not need a main loop.
==================
*/
void OGTS_StartEA(OGTS_pref_t *prefs, char mode, OGTS_callback callback)
{
  int time=0, beginning_time=0,old_time=0;
  int i=0,j=0,num_eas=-1;
  int eas[8];
  int res=-1;
  OGTS_point_t *couple = calloc(1,sizeof(OGTS_point_t));
  float factor = 0;
  int condition = -1;

  for (j=0; j<8; j++) {
    if (prefs->EA & OGTS_EAs[j]) {eas[i]=j; i++;}
  }

  num_eas=i;

  if (num_eas <0) 
    OGTS_Error("You must provide at least one EA");

  if (OGTS_IsEnabled(OGTS_VERBOSE))
      OGTS_Printf("%d EAs selected\n",num_eas);

  time=beginning_time=old_time=OGTS_GetTicks();

  i=0; j=0;
  stopacq = 0;
  
  while ((condition < 0) && (stopacq == 0)) {

    for (i=0; i < num_eas;i++) {

      if (OGTS_GetMode() == OGTS_SIMUMODE) {
	res = 0.5 *255;
      }

      else {
	res = OGTS_ReadEA(eas[i]);
      }

      if (OGTS_IsEnabled(OGTS_VERBOSE)) {
	OGTS_Printf("<- Orphy says (EA%d): %d\n",eas[i],res);
	OGTS_Printf("   time: %d\n", OGTS_GetTicks()-beginning_time);
      }

      factor = OGTS_EA_factors[eas[i]];
		
      time=OGTS_GetTicks();
      couple->x = (float)(time-beginning_time)/1000;
      couple->y = (float)(factor*((float)res/255));
      callback((void *)couple,(void *)(&eas[i]));
      j++;
      
      switch (mode) {
      case OGTS_NUM: condition = j-prefs->numacq; break;
      case OGTS_TIME: condition = OGTS_GetTicks()-beginning_time-(prefs->total_time*1000); break;
      case OGTS_KEY: break;
      default: break;
      }
    }

    time=OGTS_GetTicks(); /* added */    

    if ((prefs->dt >0)&&(prefs->dt > (time-old_time))) {

      if (OGTS_IsEnabled(OGTS_VERBOSE)) {
	OGTS_Printf("(delaying %d ms, because time-oldtime=%d (now, time=%d))\n",prefs->dt-time+old_time,time-old_time,OGTS_GetTicks()-beginning_time);
      }

      OGTS_Delay(prefs->dt-time+old_time);

    }

    else {
      if (OGTS_IsEnabled(OGTS_VERBOSE)) {
	OGTS_Printf("(not delaying: time-old_time=%d)\n",time-old_time);
      }
    }
    old_time=OGTS_GetTicks();
  }

  OGTS_Printf("Completed %d measures in %d ms\n",j,time-beginning_time);
  free (couple);
}

/* stops a running aquisition managed by OGTS_StartEA */
void OGTS_StopEA()
{
  stopacq = 1;
}

void OGTS_StartEA2(int EA,int ds,char mode, int param, OGTS_callback callback, void*param2)
{
  int time=0, beginning_time=0,old_time=0;
  int res=-1; int j=0;
  OGTS_point_t *couple = calloc(1,sizeof(OGTS_point_t));
  float factor = OGTS_EA_factors[EA];
  int condition = -1;

  time=beginning_time=old_time=OGTS_GetTicks();

  while (condition < 0) {

    res = OGTS_ReadEA(EA);
    OGTS_Printf("<- Orphy says: %d\n",res);
    
    time=OGTS_GetTicks();
    couple->x = (float)(time-beginning_time)/1000;
    couple->y = (float)(factor*((float)res/255));
    callback((void*)param2,(void *)couple);
    j++;
    //printf("Total time: %d ms\n\n",time-old_time);
    if ((ds >0)&&(ds > (time-old_time))) {
      OGTS_Delay(ds-time+old_time);
    }
    old_time=OGTS_GetTicks();

    switch (mode) {
    case OGTS_NUM: condition = j-param; break;
    case OGTS_TIME: condition = OGTS_GetTicks()-beginning_time-param; break;
    case OGTS_KEY: break;
    default: break;
    }
  }

  OGTS_Printf("Completed %d mesures in %d ms (average 1 mes. every %d ms)\n",j,time-beginning_time,(int)floorf((float)(time-beginning_time)/(float)(j))   );
  free (couple);
}
